from __future__ import annotations
from typing import TypedDict


class AlbumComment(TypedDict):
    albumId: int
    albumPaid: bool
    albumUid: int
    anchorLiked: bool
    auditStatus: int
    commentId: int
    content: str
    createdAt: int
    isHighQuality: bool
    likeStatus: int
    liked: bool
    likes: int
    nickname: str
    playProgress: int
    recStatus: int
    region: str
    replyCount: int
    smallHeader: str
    uid: int
    updatedAt: int
    userTags: list[UserTag]
    vLogoType: int
    vipIconUrl: str
    vipJumpUrl: str


class AlbumInfo(TypedDict):
    albumId: int
    albumPlayCount: int
    albumTrackCount: int
    albumCoverPath: str
    albumTitle: str
    albumUserNickName: str
    anchorId: int
    anchorGrade: int
    mvpGrade: int
    isDeleted: bool
    isPaid: bool
    isFinished: int
    anchorUrl: str
    albumUrl: str
    intro: str
    vipType: int
    logoType: int
    subscriptInfo: dict[{'albumSubscriptValue': int, 'url': str}]
    albumSubscript: int


class FollowingInfo(TypedDict):
    uid: int
    coverPath: str
    anchorNickName: str
    background: str
    description: str
    url: str
    grade: int
    mvpGrade: int
    gradeType: int
    trackCount: int
    albumCount: int
    followerCount: int
    followingCount: int
    isFollow: bool
    beFollow: bool
    isBlack: bool
    logoType: int


class PubInfo(TypedDict):
    id: int
    title: str
    subTitle: str
    coverPath: str
    isFinished: bool
    isPaid: bool
    anchorUrl: str
    anchorNickname: str
    anchorUid: int
    playCount: int
    trackCount: int
    albumUrl: str
    description: str
    vipType: int
    albumSubscript: int


class SubscriptionInfo(TypedDict):
    id: int
    title: str
    subTitle: str
    description: str
    coverPath: str
    isFinished: bool
    isPaid: bool
    anchor: dict[{'anchorUrl': str, 'anchorNickName': str, 'anchorUid': int, 'anchorCoverPath': str, 'logoType': int}]
    playCount: int
    trackCount: int
    albumUrl: str
    albumStatus: int
    lastUptrackAt: int
    lastUptrackAtStr: str
    serialState: int
    isTop: bool
    categoryCode: str
    categoryTitle: str
    lastUptrackUrl: str
    lastUptrackTitle: str
    vipType: int
    albumSubscript: int
    albumScore: str


class TrackInfo(TypedDict):
    trackId: int
    title: str
    trackUrl: str
    coverPath: str
    createTimeAsString: str
    albumId: int
    albumTitle: str
    albumUrl: str
    anchorUid: int
    anchorUrl: str
    nickname: str
    durationAsString: str
    playCount: int
    showLikeBtn: bool
    isLike: bool
    isPaid: bool
    isRelay: bool
    showDownloadBtn: bool
    showCommentBtn: bool
    showForwardBtn: bool
    isVideo: bool
    videoCover: str
    breakSecond: int
    length: int
    isAlbumShow: bool


class UserBasicInfo(TypedDict):
    uid: int
    nickName: str
    cover: str
    background: str
    isVip: bool
    constellationType: int
    personalSignature: str
    personalDescription: str
    fansCount: int
    gender: int
    birthMonth: int
    birthDay: int
    province: str
    city: str
    anchorGrade: int
    mvpGrade: int
    anchorGradeType: int
    isMusician: bool
    anchorUrl: str
    relation: dict[{'isFollow': bool, 'beFollow': bool, 'isBlack': bool}] | None
    liveInfo: dict[{'id': int, 'roomId': int, 'coverPath': str, 'liveUrl': str, 'status': int}] | None
    logoType: int
    followingCount: int
    tracksCount: int
    albumsCount: int
    albumCountReal: int
    userCompany: str
    qualificationGuideInfos: list[str]


class UserDetailedInfo(TypedDict):
    uid: int
    pubPageInfo: dict[{'totalCount': int, 'pubInfoList': list[PubInfo]}]
    trackPageInfo: dict[{'totalCount': int, 'trackInfoList': list[TrackInfo]}]
    subscriptionPageInfo: dict[{'privateSub': bool, 'totalCount': int, 'subscribeInfoList': list[SubscriptionInfo]}]
    followingPageInfo: dict[{'totalCount': int, 'followInfoList': list[FollowingInfo]}]


class UserTag(TypedDict):
    businessName: str
    businessType: int
    extra: dict
    height: int
    icon: str
    jumpUrl: str
    scale: int
    width: int



from datetime import datetime

from ximalaya.client import XimalayaClient, ResponsePaginator
from ximalaya.typing import AlbumComment


def api_album_comment_list(client: XimalayaClient, album_id: int, page_size: int = 50, order: str = 'content-score-desc') -> ResponsePaginator[AlbumComment]:
    client.host = 'mobile.ximalaya.com'
    timestamp = int(datetime.now().timestamp() * 1000)

    return ResponsePaginator(
        client,
        url_path=f'/album-comment-mobile/web/album/comment/list/query/{timestamp}?albumId={album_id}&order={order}&pageSize={page_size}',
        page_name='pageId',
        data_path='data.comments.list'
    )



from ximalaya.client import XimalayaClient, ResponsePaginator
from ximalaya.typing import FollowingInfo, UserBasicInfo, UserDetailedInfo, SubscriptionInfo


def api_user_following(client: XimalayaClient, uid: int, page_size: int = 50) -> ResponsePaginator[FollowingInfo]:
    client.host = 'www.ximalaya.com'

    return ResponsePaginator(
        client,
        url_path=f'/revision/user/following?uid={uid}&pageSize={page_size}&keyWord=',
        data_path='data.followingsPageInfo'
    )


def api_user_basic(client: XimalayaClient, uid: int) -> UserBasicInfo:
    client.host = 'www.ximalaya.com'

    return client.get(f'/revision/user/basic?uid={uid}')['data']


def api_user(client: XimalayaClient, uid: int) -> UserDetailedInfo:
    client.host = 'www.ximalaya.com'

    return client.get(f'/revision/user?uid={uid}')['data']


def api_user_sub(client: XimalayaClient, uid: int, page_size: int = 10) -> ResponsePaginator[SubscriptionInfo]:
    client.host = 'www.ximalaya.com'

    return ResponsePaginator(
        client,
        url_path=f'/revision/user/sub?pageSize={page_size}&keyWord=&uid={uid}',
        data_path='data.albumsInfo'
    )



import json
import logging
import urllib.request
from typing import Generic, Iterator, TypeVar

from jsonpath_ng import parse


class XimalayaClient:
    host: str

    def __init__(self, host: str = 'www.ximalaya.com', headers: dict = None):
        self.host = host

        self.headers = {} if headers is None else headers

        if 'User-Agent' not in self.headers:
            self.headers['User-Agent'] = ''

    def get(self, path: str, method: str = 'GET'):
        url = f'https://{self.host}/{path.lstrip("/")}'

        logging.debug('[XimalayaClient][GET]: %s', url)

        req = urllib.request.Request(url, headers=self.headers, method=method)

        with urllib.request.urlopen(req) as fp:
            return json.loads(fp.read().decode('utf-8'))


T = TypeVar('T')


class ResponsePaginator(Generic[T], Iterator):
    client: XimalayaClient
    url_path: str
    page_name: str
    page_num: int

    def __init__(self, client: XimalayaClient, url_path: str, data_path: str, page_name: str = 'page', page_num: int = 1):
        self.client = client
        self.url_path = url_path
        self.page_name = page_name
        self.page_num = page_num
        self.jsonpath_expression = parse(f'$.{data_path}')

    def __next__(self) -> list[T]:
        json_data = self.client.get(f'{self.url_path}&{self.page_name}={self.page_num}')

        match = self.jsonpath_expression.find(json_data)

        if match is None or len(match[0].value) == 0:
            raise StopIteration()

        self.page_num += 1

        return match[0].value



from ximalaya.client import XimalayaClient, ResponsePaginator
from ximalaya.typing import AlbumInfo


def api_category_v2_albums(client: XimalayaClient, category_id: int, page_size: int = 50, sort_by: int = 1) -> ResponsePaginator[AlbumInfo]:
    client.host = 'www.ximalaya.com'

    return ResponsePaginator(
        client,
        url_path=f'/revision/category/v2/albums?pageSize={page_size}&sort={sort_by}&categoryId={category_id}',
        page_name='pageNum',
        data_path='data.albums'
    )
